# coding=utf-8


def gen_statistics_data(extract_function, sample_dir, tag=""):
    keyword_count_file = "keyword_count_%s.csv" % tag
    keywords_file = "keywords_%s.txt" % tag
    keywords_with_year = "keywords_with_year_%s.txt" % tag
    paper_count_by_year = "paper_count_by_year_%s.csv" % tag
    author_location = "author_location_%s.txt" % tag

    pcby = open(paper_count_by_year, "w+")
    kf = open(keywords_file, "w+")
    kwy = open(keywords_with_year, "w+")
    kcf = open(keyword_count_file, "w+")
    al = open(author_location, "w+")
    keyword_dict = {}
    keyword_year_dict = {}

    for year, keywords, location in apply(extract_function, (sample_dir,)):
        for keyword in keywords:
            kf.write(keyword + "|")
            kwy.write(keyword + "|")

            if keyword not in keyword_dict:
                keyword_dict[keyword] = 1
            else:
                keyword_dict[keyword] += 1

            if year not in keyword_year_dict:
                keyword_year_dict[year] = 1
            else:
                keyword_year_dict[year] += 1

        kf.write("\n")
        kwy.write(year + "\n")
        if location is not None:
            al.write(location+"\n")

    kf.close()
    kwy.close()
    al.close()

    # sort key word
    sorted_keyword_dict = sorted(keyword_dict.iteritems(), key=lambda d: d[1], reverse=True)
    for item in sorted_keyword_dict:
        kcf.write("%s,%d\n" % (item[0], item[1]))

    kcf.close()

    sorted_keyword_year_dict = sorted(keyword_year_dict.iteritems(), key=lambda d: d[0], reverse=False)
    for item in sorted_keyword_year_dict:
        pcby.write("%s,%d\n" % (item[0], item[1]))
    pcby.close()






# coding=utf-8
import math

if __name__ == '__main__':
    java_distance = {}
    python_distance = {}

    for line in open("java_distance.csv"):
        parts = line.strip().split(",")
        python_distance[",".join(sorted([parts[0], parts[1]]))] = parts[2]

    for line in open("python_distance_lan.csv"):
        parts = line.strip().split(",")
        java_distance[",".join(sorted([parts[0], parts[1]]))] = parts[2]

    count_dict = {}
    for key, python_dist in python_distance.items():
        java_dist = java_distance[key]

        diff = int(math.fabs(int(python_dist) - int(java_dist)))
        if str(diff) in count_dict:
            count_dict[str(diff)] += 1
        else:
            count_dict[str(diff)] = 0

    count_dict_sorted = sorted(count_dict.iteritems(), key=lambda d: int(d[0]), reverse=False)
    total = sum(count_dict.values())
    for item in count_dict_sorted:
        print "%s,%s,%.3f%%" % (item[0], item[1], 1.0 * int(item[1])/total*100)





# coding=utf-8
import requests
from selenium import webdriver


def get_headless_chrome():
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--headless')
    return webdriver.Chrome(chrome_options=chrome_options)


def form_browser_headers():
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.84 Safari/537.36',
        'Connection': 'keep-alive', 'Cache-Control': 'max-age=0', 'Content-Type': 'application/x-www-form-urlencoded;',
        'Upgrade-Insecure-Requests': '1', 'Accept-Language': 'zh-cn,zh;q=0.8,en-us;q=0.5,en;q=0.3'}
    return headers


def fake_browser():
    return requests.Session()


if __name__ == '__main__':
    browser = fake_browser()
    headers = form_browser_headers()
    browser.get("http://A0000051.casmart.com.cn", headers=headers)
    response = browser.get("http://A0000277.casmart.com.cn/IndexHandler.ashx?action=HomeProduct&showNum=16&timer=0.4323412121412289")
    print response.text



import datetime
import time
import subprocess
import os
import signal
from subprocess import STDOUT, check_output


def run_cmd(cmd, timeout):
    start = datetime.datetime.now()
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    if timeout != 0:
        while p.poll() is None:
            time.sleep(1)
            now = datetime.datetime.now()
            if (now - start).seconds > timeout:
                os.kill(p.pid, signal.SIGKILL)
                os.waitpid(-1, os.WNOHANG)
                return [None, None]
    stdout, stderr = p.communicate()
    return [stdout, stderr]


if __name__ == '__main__':
    stdout, stderr = run_cmd("ls", 10)
    print "stdout: %s" % stdout
    print "stderr: %s" % stderr



import subprocess
import threading
import psutil


class CommandRunner(object):
    def __init__(self, cmd, timeout):
        self.cmd = cmd
        self.process = None
        self.stdout = None
        self.stderr = None
        self.timeout = timeout
 
    def run(self):
        def target():
            self.process = subprocess.Popen(self.cmd, shell=True,
                                            stderr=subprocess.PIPE,
                                            stdout=subprocess.PIPE)
            self.stdout, self.stderr = self.process.communicate()
 
        thread = threading.Thread(target=target)
        thread.setDaemon(True)
        thread.start()
 
        thread.join(self.timeout)
        if thread.is_alive():
            print 'Timeout! Terminating process!'
            p = psutil.Process(int(self.process.pid))
            for p_child in p.children():
                p_child.terminate()
            self.process.terminate()
            return -1, 'Timeout! Terminating process!'
        return self.process.returncode, self.stdout
  

if __name__ == "__main__":
    cmd_line = 'python sleep.py'
    cmd = CommandRunner(cmd_line, 11)
    code, output_size = cmd.run()
    print "%d(%s)" % (code, output_size)



import os
import random
import time


class FileLock:
    def __init__(self, filename):
        self.filename = filename
        self.fd = None
        self.pid = os.getpid()

    def acquire(self):
        try:
            self.fd = os.open(self.filename, os.O_CREAT|os.O_EXCL|os.O_RDWR)
            # Only needed to let readers know who's locked the file
            os.write(self.fd, "%d" % self.pid)
            return 1    # return ints so this can be used in older Pythons
        except OSError:
            self.fd = None
            return 0

    def release(self):
        if not self.fd:
            return 0
        try:
            os.close(self.fd)
            os.remove(self.filename)
            return 1
        except OSError:
            return 0

    def __del__(self):
        self.release()


def main():
    lock = FileLock("lock.file")
    while 1:
        if lock.acquire():
            print("acquired lock.")
            time.sleep(random.randint(1, 3))
            lock.release()
            print("released lock.")
            time.sleep(random.randint(2, 3))
        else:
            print("Unable to acquire lock.")
            time.sleep(1)


if __name__ == "__main__":
    main()




# coding=utf-8

from urlparse import urlparse
import re
import os
from browser_utils import fake_browser, form_browser_headers
import sys
from sqlite_utils import sqlite_get_db, sqlite_excute, sqlite_query
from string_utils import get_links, gen_md5
import logging

reload(sys)
sys.setdefaultencoding('utf-8')


def check_if_existed(conn, url_md5):
    # check url_md5 sql
    count_query = "SELECT COUNT(*) FROM URL WHERE url_md5='%s'" % url_md5
    for row in sqlite_query(conn, count_query):
        return row[0] > 0

    return False


def add_new_url(conn, urls):
    for url in urls:
        url_md5 = gen_md5(url)
        if check_if_existed(conn, url_md5):
            continue
        insert_sql = "INSERT INTO URL VALUES('%s', '%s', 0);" % (url, url_md5)
        # print "add new url: %s" % url
        sqlite_excute(conn, insert_sql)


def set_url_status(conn, url_md5, status):
    update_sql = "UPDATE URL SET url_status=%d WHERE url_md5='%s';" % (status, url_md5)
    sqlite_excute(conn, update_sql)


def get_url_to_download(conn, count):
    count_query = "SELECT url FROM URL where url_status=0 LIMIT %d" % count
    urls = []
    for row in sqlite_query(conn, count_query):
        urls.append(row[0])
    return urls


def init_db(db_tag, reset_db):
    db_name = "%s.db" % db_tag
    if reset_db and os.path.exists(db_name):
        os.remove(db_name)
    conn = sqlite_get_db(db_name)
    create_table_sql = '''
                   CREATE TABLE IF NOT EXISTS URL
                   (url          TEXT     NOT NULL,
                   url_md5       TEXT     PRIMARY KEY NOT NULL,
                   url_status    INTEGER  NOT NULL
                    ) ;
                '''
    sqlite_excute(conn, create_table_sql)
    return conn


def simple_crawl(tag, host, seed_url, reset_db, call_back):
    conn = init_db(tag, reset_db)
    add_new_url(conn, seed_url)

    browser = fake_browser()
    headers = form_browser_headers()

    while True:
        urls = get_url_to_download(conn, 10)
        if len(urls) == 0:
            print "nothing todo, all job done!!!"
            logging.error("nothing todo, all job done!!!")
            break
        for url in urls:
            print "crawl url: %s" % url
            response = browser.get(url, headers=headers)
            new_urls = get_links(host, url, response.text)
            refined_urls = apply(call_back, (url, new_urls, response.text))
            add_new_url(conn, refined_urls)
            set_url_status(conn, gen_md5(url), 1)


if __name__ == '__main__':
    tag = "casmart"
    host = "casmart.com.cn"
    seed_url = ["http://www.casmart.com.cn/product/supplierlist.aspx"]



# coding=utf-8

import sqlite3


def sqlite_get_db(db_name):
    return sqlite3.connect(db_name)


def sqlite_excute(conn, sql):
    c = conn.cursor()
    c.execute(sql)
    conn.commit()


def sqlite_query(conn, sql):
    c = conn.cursor()
    cursor = c.execute(sql)
    for row in cursor:
        yield row


if __name__ == '__main__':
    conn = sqlite_get_db("test.db")
    # create table
    create_table_sql = '''
       CREATE TABLE IF NOT EXISTS COMPANY
       (ID INTEGER PRIMARY KEY    AUTOINCREMENT,
       address           TEXT    NOT NULL,
       address_md5       TEXT    NOT NULL,
       
        ) ;
    '''

    sqlite_excute(conn, create_table_sql)

    insert_sql = "INSERT INTO COMPANY VALUES(NULL, 'url', 'url_md5');"
    sqlite_excute(conn, insert_sql)

    # test query
    count_query = "SELECT COUNT(*) FROM COMPANY WHERE url='url'"

    for row in sqlite_query(conn, count_query):
        print row[0]